@@ -1,9 +1,7 @@
 use std::fmt;
 use std::error;
 
-use {
-    TokenType,
-};
+use TokenType;
 
 
 /// An XML parser errors.
@@ -19,6 +17,17 @@ pub enum Error {
     UnknownToken(TextPos),
 }
 
+impl Error {
+    /// Returns the error position.
+    pub fn pos(&self) -> TextPos {
+        match *self {
+            Error::InvalidToken(_, pos, _) => pos,
+            Error::UnexpectedToken(_, pos) => pos,
+            Error::UnknownToken(pos) => pos,
+        }
+    }
+}
+
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
@@ -61,6 +70,9 @@ pub enum StreamError {
     /// An unknown token.
     InvalidName,
 
+    /// An attribute value must not contain the `<` character.
+    InvalidAttributeValue,
+
     /// An invalid/unexpected character.
     ///
     /// The first byte is an actual one, others - expected.
@@ -76,7 +88,7 @@ pub enum StreamError {
     /// Includes: `' ' \n \r \t &#x20; &#x9; &#xD; &#xA;`.
     InvalidSpace(char, TextPos),
 
-    /// An unexpected character instead of an XML space.
+    /// An unexpected string.
     ///
     /// The first string is an actual one, others - expected.
     ///
@@ -99,6 +111,9 @@ impl fmt::Display for StreamError {
             StreamError::InvalidName => {
                 write!(f, "invalid name token")
             }
+            StreamError::InvalidAttributeValue => {
+                write!(f, "attribute value with '<' character is not allowed")
+            }
             StreamError::InvalidChar(ref chars, pos) => {
                 // Vec<u8> -> Vec<String>
                 let list: Vec<String> =
@@ -147,7 +162,7 @@ pub struct TextPos {
 impl TextPos {
     /// Constructs a new `TextPos`.
     ///
-    /// Should not be invoked manually, but rather via `Stream::gen_error_pos`.
+    /// Should not be invoked manually, but rather via `Stream::gen_text_pos`.
     pub fn new(row: u32, col: u32) -> TextPos {
         TextPos { row, col }
     }
@@ -158,13 +173,3 @@ impl fmt::Display for TextPos {
         write!(f, "{}:{}", self.row, self.col)
     }
 }
-
-#[test]
-fn err_size_1() {
-    assert!(::std::mem::size_of::<Error>() <= 64);
-}
-
-#[test]
-fn err_size_2() {
-    assert!(::std::mem::size_of::<StreamError>() <= 64);
-}
