@@ -35,7 +35,7 @@ If you are looking for a more high-level solution - checkout
 - Good error processing. All error types contain position (line:column) where it occurred.
 - No heap allocations.
 - No dependencies.
-- Tiny. ~1500 LOC and ~35KiB in the release build according to the `cargo-bloat`.
+- Tiny. ~1500 LOC and ~40KiB in the release build according to the `cargo-bloat`.
 
 ## Limitations
 
@@ -56,7 +56,7 @@ If you are looking for a more high-level solution - checkout
 
 #![cfg_attr(feature = "cargo-clippy", allow(unreadable_literal))]
 
-#![doc(html_root_url = "https://docs.rs/xmlparser/0.8.0")]
+#![doc(html_root_url = "https://docs.rs/xmlparser/0.9.0")]
 
 #![forbid(unsafe_code)]
 #![warn(missing_docs)]
@@ -64,11 +64,24 @@ If you are looking for a more high-level solution - checkout
 
 use std::fmt;
 
+
+macro_rules! matches {
+    ($expression:expr, $($pattern:tt)+) => {
+        match $expression {
+            $($pattern)+ => true,
+            _ => false
+        }
+    }
+}
+
+
+mod bytestream;
 mod error;
 mod stream;
 mod strspan;
 mod xmlchar;
 
+pub use bytestream::*;
 pub use error::*;
 pub use stream::*;
 pub use strspan::*;
@@ -76,75 +89,187 @@ pub use xmlchar::*;
 
 
 /// An XML token.
+#[allow(missing_docs)]
 #[derive(Clone, Copy, PartialEq, Debug)]
 pub enum Token<'a> {
     /// Declaration token.
     ///
-    /// Version, encoding and standalone.
-    ///
-    /// Example: `<?xml version="1.0"?>`
-    Declaration(StrSpan<'a>, Option<StrSpan<'a>>, Option<bool>),
+    /// ```text
+    /// <?xml version='1.0' encoding='UTF-8' standalone='yes'?>
+    ///                ---                                      - version
+    ///                               -----                     - encoding?
+    ///                                                  ---    - standalone?
+    /// ------------------------------------------------------- - span
+    /// ```
+    Declaration {
+        version: StrSpan<'a>,
+        encoding: Option<StrSpan<'a>>,
+        standalone: Option<bool>,
+        span: StrSpan<'a>,
+    },
+
     /// Processing instruction token.
     ///
-    /// Example: `<?target content?>`
-    ProcessingInstruction(StrSpan<'a>, Option<StrSpan<'a>>),
-    /// The comment token.
+    /// ```text
+    /// <?target content?>
+    ///   ------           - target
+    ///          -------   - content?
+    /// ------------------ - span
+    /// ```
+    ProcessingInstruction {
+        target: StrSpan<'a>,
+        content: Option<StrSpan<'a>>,
+        span: StrSpan<'a>,
+    },
+
+    /// Comment token.
     ///
-    /// Example: `<!-- text -->`
-    Comment(StrSpan<'a>),
+    /// ```text
+    /// <!-- text -->
+    ///     ------    - text
+    /// ------------- - span
+    /// ```
+    Comment {
+        text: StrSpan<'a>,
+        span: StrSpan<'a>,
+    },
+
     /// DOCTYPE start token.
     ///
-    /// Example: `<!DOCTYPE note [`
-    DtdStart(StrSpan<'a>, Option<ExternalId<'a>>),
+    /// ```text
+    /// <!DOCTYPE greeting SYSTEM "hello.dtd" [
+    ///           --------                      - name
+    ///                    ------------------   - external_id?
+    /// --------------------------------------- - span
+    /// ```
+    DtdStart {
+        name: StrSpan<'a>,
+        external_id: Option<ExternalId<'a>>,
+        span: StrSpan<'a>,
+    },
+
     /// Empty DOCTYPE token.
     ///
-    /// Example: `<!DOCTYPE note>`
-    EmptyDtd(StrSpan<'a>, Option<ExternalId<'a>>),
+    /// ```text
+    /// <!DOCTYPE greeting SYSTEM "hello.dtd">
+    ///           --------                     - name
+    ///                    ------------------  - external_id?
+    /// -------------------------------------- - span
+    /// ```
+    EmptyDtd {
+        name: StrSpan<'a>,
+        external_id: Option<ExternalId<'a>>,
+        span: StrSpan<'a>,
+    },
+
     /// ENTITY token.
     ///
     /// Can appear only inside the DTD.
     ///
-    /// Example: `<!ENTITY ns_extend "http://test.com">`
-    EntityDeclaration(StrSpan<'a>, EntityDefinition<'a>),
+    /// ```text
+    /// <!ENTITY ns_extend "http://test.com">
+    ///          ---------                    - name
+    ///                     ---------------   - definition
+    /// ------------------------------------- - span
+    /// ```
+    EntityDeclaration {
+        name: StrSpan<'a>,
+        definition: EntityDefinition<'a>,
+        span: StrSpan<'a>,
+    },
+
     /// DOCTYPE end token.
     ///
-    /// Example: `]>`
-    DtdEnd,
+    /// ```text
+    /// <!DOCTYPE svg [
+    ///    ...
+    /// ]>
+    /// -- - span
+    /// ```
+    DtdEnd {
+        span: StrSpan<'a>,
+    },
+
     /// Element start token.
     ///
-    /// Contains prefix and local part of the qualified name.
+    /// ```text
+    /// <ns:elem attr="value"/>
+    ///  --                     - prefix
+    ///     ----                - local
+    /// --------                - span
+    /// ```
+    ElementStart {
+        prefix: StrSpan<'a>,
+        local: StrSpan<'a>,
+        span: StrSpan<'a>,
+    },
+
+    /// Attribute token.
     ///
-    /// Example: `<elem`
-    ElementStart(StrSpan<'a>, StrSpan<'a>),
-    /// Attribute.
-    ///
-    /// Contains prefix and local part of the qualified name and value.
-    ///
-    /// Example: `name="value"`
-    Attribute((StrSpan<'a>, StrSpan<'a>), StrSpan<'a>),
+    /// ```text
+    /// <elem ns:attr="value"/>
+    ///       --              - prefix
+    ///          ----         - local
+    ///                -----  - value
+    ///       --------------- - span
+    /// ```
+    Attribute {
+        prefix: StrSpan<'a>,
+        local: StrSpan<'a>,
+        value: StrSpan<'a>,
+        span: StrSpan<'a>,
+    },
+
     /// Element end token.
-    ElementEnd(ElementEnd<'a>),
+    ///
+    /// ```text
+    /// <ns:elem>text</ns:elem>
+    ///                         - ElementEnd::Open
+    ///         -               - span
+    /// ```
+    ///
+    /// ```text
+    /// <ns:elem>text</ns:elem>
+    ///                -- ----  - ElementEnd::Close(prefix, local)
+    ///              ---------- - span
+    /// ```
+    ///
+    /// ```text
+    /// <ns:elem/>
+    ///                         - ElementEnd::Empty
+    ///         --              - span
+    /// ```
+    ElementEnd {
+        end: ElementEnd<'a>,
+        span: StrSpan<'a>,
+    },
+
     /// Text token.
     ///
     /// Contains text between elements including whitespaces.
     /// Basically everything between `>` and `<`.
     ///
-    /// Contains text as is. Use [`TextUnescape`] to unescape it.
+    /// ```text
+    /// <p> text </p>
+    ///    ------     - text
+    /// ```
     ///
-    /// Example: `<text>text</text>`
-    ///
-    /// [`TextUnescape`]: struct.TextUnescape.html
-    Text(StrSpan<'a>),
-    /// Whitespaces token.
-    ///
-    /// The same as `Text` token, but contains only spaces.
-    ///
-    /// Spaces can be encoded like `&#x20`.
-    Whitespaces(StrSpan<'a>),
+    /// The token span is equal to the `text`.
+    Text {
+        text: StrSpan<'a>,
+    },
+
     /// CDATA token.
     ///
-    /// Example: `<![CDATA[text]]>`
-    Cdata(StrSpan<'a>),
+    /// ```text
+    /// <p><![CDATA[text]]></p>
+    ///             ----        - text
+    ///    ----------------     - span
+    /// ```
+    Cdata {
+        text: StrSpan<'a>,
+        span: StrSpan<'a>,
+    },
 }
 
 
@@ -294,7 +419,7 @@ impl<'a> Tokenizer<'a> {
     ///
     /// By default, `xmlparser` will check for DTD, root element, etc.
     /// But if we have to parse an XML fragment, it will lead to an error.
-    /// This method switch the parser to the root element content parsing mode.
+    /// This method switches the parser to the root element content parsing mode.
     /// So it will treat any data as a content of the root element.
     pub fn enable_fragment_mode(&mut self) {
         self.state = State::Elements;
@@ -374,7 +499,7 @@ impl<'a> Tokenizer<'a> {
             State::Dtd => {
                 let token_type = parse_token_type!();
                 match token_type {
-                    TokenType::ElementDecl
+                      TokenType::ElementDecl
                     | TokenType::NotationDecl
                     | TokenType::AttlistDecl => {
                         if Self::consume_decl(s).is_err() {
@@ -393,7 +518,7 @@ impl<'a> Tokenizer<'a> {
                         Self::parse_pi(s)
                     }
                     TokenType::DoctypeEnd => {
-                        Ok(Token::DtdEnd)
+                        Ok(Token::DtdEnd { span: s.slice_back(s.pos() - 2) })
                     }
                     TokenType::Whitespace => {
                         s.skip_spaces();
@@ -578,70 +703,79 @@ impl<'a> Tokenizer<'a> {
         map_err_at!(Self::parse_declaration_impl(s), TokenType::XMLDecl, s, -6)
     }
 
+    // XMLDecl ::= '<?xml' VersionInfo EncodingDecl? SDDecl? S? '?>'
     fn parse_declaration_impl(s: &mut Stream<'a>) -> StreamResult<Token<'a>> {
+        let start = s.pos() - 6;
+
         let version = Self::parse_version_info(s)?;
         let encoding = Self::parse_encoding_decl(s)?;
         let standalone = Self::parse_standalone(s)?;
 
-        s.skip_ascii_spaces();
+        s.skip_spaces();
         s.skip_string(b"?>")?;
 
-        Ok(Token::Declaration(version, encoding, standalone))
+        let span = s.slice_back(start);
+
+        Ok(Token::Declaration { version, encoding, standalone, span })
     }
 
+    // VersionInfo ::= S 'version' Eq ("'" VersionNum "'" | '"' VersionNum '"')
+    // VersionNum  ::= '1.' [0-9]+
     fn parse_version_info(s: &mut Stream<'a>) -> StreamResult<StrSpan<'a>> {
-        s.skip_ascii_spaces();
+        s.skip_spaces();
         s.skip_string(b"version")?;
         s.consume_eq()?;
-        s.consume_quote()?;
+        let quote = s.consume_quote()?;
 
         let start = s.pos();
         s.skip_string(b"1.")?;
         s.skip_bytes(|_, c| c.is_xml_digit());
         let ver = s.slice_back(start);
 
-        s.consume_quote()?;
+        s.consume_byte(quote)?;
 
         Ok(ver)
     }
 
-    // S 'encoding' Eq ('"' EncName '"' | "'" EncName "'" )
+
+    // EncodingDecl ::= S 'encoding' Eq ('"' EncName '"' | "'" EncName "'" )
+    // EncName      ::= [A-Za-z] ([A-Za-z0-9._] | '-')*
     fn parse_encoding_decl(s: &mut Stream<'a>) -> StreamResult<Option<StrSpan<'a>>> {
-        s.skip_ascii_spaces();
+        s.skip_spaces();
 
         if s.skip_string(b"encoding").is_err() {
             return Ok(None);
         }
 
         s.consume_eq()?;
-        s.consume_quote()?;
+        let quote = s.consume_quote()?;
         // [A-Za-z] ([A-Za-z0-9._] | '-')*
         // TODO: check that first byte is [A-Za-z]
         let name = s.consume_bytes(|_, c| {
-            c.is_xml_letter()
-                || c.is_xml_digit()
-                || c == b'.'
-                || c == b'-'
-                || c == b'_'
+               c.is_xml_letter()
+            || c.is_xml_digit()
+            || c == b'.'
+            || c == b'-'
+            || c == b'_'
         });
-        s.consume_quote()?;
+        s.consume_byte(quote)?;
 
         Ok(Some(name))
     }
 
-    // S 'standalone' Eq (("'" ('yes' | 'no') "'") | ('"' ('yes' | 'no') '"'))
+    // SDDecl ::= S 'standalone' Eq (("'" ('yes' | 'no') "'") | ('"' ('yes' | 'no') '"'))
     fn parse_standalone(s: &mut Stream<'a>) -> StreamResult<Option<bool>> {
-        s.skip_ascii_spaces();
+        s.skip_spaces();
 
         if s.skip_string(b"standalone").is_err() {
             return Ok(None);
         }
 
         s.consume_eq()?;
-        s.consume_quote()?;
+        let quote = s.consume_quote()?;
 
         let start = s.pos();
-        let value = s.consume_name()?.to_str();
+        let value = s.consume_name()?.as_str();
 
         let flag = match value {
             "yes" => true,
@@ -653,38 +787,38 @@ impl<'a> Tokenizer<'a> {
             }
         };
 
-        s.consume_quote()?;
+        s.consume_byte(quote)?;
 
         Ok(Some(flag))
     }
 
-    // '<!--' ((Char - '-') | ('-' (Char - '-')))* '-->'
     fn parse_comment(s: &mut Stream<'a>) -> Result<Token<'a>> {
         let start = s.pos() - 4;
+        Self::parse_comment_impl(s)
+            .map_err(|_| Error::InvalidToken(TokenType::Comment, s.gen_text_pos_from(start), None))
+    }
+
+    // '<!--' ((Char - '-') | ('-' (Char - '-')))* '-->'
+    fn parse_comment_impl(s: &mut Stream<'a>) -> StreamResult<Token<'a>> {
+        let start = s.pos() - 4;
 
         let text = s.consume_chars(|s, c| {
             if c == '-' && s.starts_with(b"-->") {
                 return false;
             }
 
-            if !c.is_xml_char() {
-                return false;
-            }
-
-            true
+            c.is_xml_char()
         });
 
-        if text.to_str().contains("--") {
-            let pos = s.gen_text_pos_from(start);
-            return Err(Error::InvalidToken(TokenType::Comment, pos, None));
+        s.skip_string(b"-->")?;
+
+        if text.as_str().contains("--") {
+            return Err(StreamError::UnexpectedEndOfStream); // Error type doesn't matter.
         }
 
-        if s.skip_string(b"-->").is_err() {
-            let pos = s.gen_text_pos_from(start);
-            return Err(Error::InvalidToken(TokenType::Comment, pos, None));
-        }
+        let span = s.slice_back(start);
 
-        Ok(Token::Comment(text))
+        Ok(Token::Comment { text, span })
     }
 
     fn parse_pi(s: &mut Stream<'a>) -> Result<Token<'a>> {
@@ -694,6 +828,8 @@ impl<'a> Tokenizer<'a> {
     // PI       ::= '<?' PITarget (S (Char* - (Char* '?>' Char*)))? '?>'
     // PITarget ::= Name - (('X' | 'x') ('M' | 'm') ('L' | 'l'))
     fn parse_pi_impl(s: &mut Stream<'a>) -> StreamResult<Token<'a>> {
+        let start = s.pos() - 2;
+
         let target = s.consume_name()?;
 
         s.skip_spaces();
@@ -718,7 +854,9 @@ impl<'a> Tokenizer<'a> {
 
         s.skip_string(b"?>")?;
 
-        Ok(Token::ProcessingInstruction(target, content))
+        let span = s.slice_back(start);
+
+        Ok(Token::ProcessingInstruction { target, content, span })
     }
 
     fn parse_doctype(s: &mut Stream<'a>) -> Result<Token<'a>> {
@@ -727,18 +865,28 @@ impl<'a> Tokenizer<'a> {
 
     // doctypedecl ::= '<!DOCTYPE' S Name (S ExternalID)? S? ('[' intSubset ']' S?)? '>'
     fn parse_doctype_impl(s: &mut Stream<'a>) -> StreamResult<Token<'a>> {
+        let start = s.pos() - 9;
+
         s.consume_spaces()?;
         let name = s.consume_name()?;
         s.skip_spaces();
 
-        let id = Self::parse_external_id(s)?;
+        let external_id = Self::parse_external_id(s)?;
         s.skip_spaces();
 
-        let c = s.consume_either(&[b'[', b'>'])?;
+        let c = s.curr_byte()?;
+        if c != b'[' && c !=  b'>' {
+            let chars = vec![c, b'[', b'>'];
+            return Err(StreamError::InvalidChar(chars, s.gen_text_pos()));
+        }
+
+        s.advance(1);
+
+        let span = s.slice_back(start);
         if c == b'[' {
-            Ok(Token::DtdStart(name, id))
+            Ok(Token::DtdStart { name, external_id, span })
         } else {
-            Ok(Token::EmptyDtd(name, id))
+            Ok(Token::EmptyDtd { name, external_id, span })
         }
     }
 
@@ -754,7 +902,7 @@ impl<'a> Tokenizer<'a> {
             let literal1 = s.consume_bytes(|_, c| c != quote);
             s.consume_byte(quote)?;
 
-            let v = if id.to_str() == "SYSTEM" {
+            let v = if id.as_str() == "SYSTEM" {
                 ExternalId::System(literal1)
             } else {
                 s.consume_spaces()?;
@@ -781,10 +929,11 @@ impl<'a> Tokenizer<'a> {
     // GEDecl      ::= '<!ENTITY' S Name S EntityDef S? '>'
     // PEDecl      ::= '<!ENTITY' S '%' S Name S PEDef S? '>'
     fn parse_entity_decl_impl(s: &mut Stream<'a>) -> StreamResult<Token<'a>> {
+        let start = s.pos() - 8;
+
         s.consume_spaces()?;
 
-        let is_ge = if s.curr_byte()? == b'%' {
-            s.consume_byte(b'%')?;
+        let is_ge = if s.try_consume_byte(b'%') {
             s.consume_spaces()?;
             false
         } else {
@@ -793,11 +942,13 @@ impl<'a> Tokenizer<'a> {
 
         let name = s.consume_name()?;
         s.consume_spaces()?;
-        let def = Self::parse_entity_def(s, is_ge)?;
+        let definition = Self::parse_entity_def(s, is_ge)?;
         s.skip_spaces();
         s.consume_byte(b'>')?;
 
-        Ok(Token::EntityDeclaration(name, def))
+        let span = s.slice_back(start);
+
+        Ok(Token::EntityDeclaration { name, definition, span })
     }
 
     // EntityDef   ::= EntityValue | (ExternalID NDataDecl?)
@@ -821,7 +972,7 @@ impl<'a> Tokenizer<'a> {
                     if is_ge {
                         s.skip_spaces();
                         if s.starts_with(b"NDATA") {
-                            s.skip_string(b"NDATA")?;
+                            s.advance(5);
                             s.consume_spaces()?;
                             s.skip_name()?;
                             // TODO: NDataDecl is not supported
@@ -857,13 +1008,17 @@ impl<'a> Tokenizer<'a> {
     // CData   ::= (Char* - (Char* ']]>' Char*))
     // CDEnd   ::= ']]>'
     fn parse_cdata_impl(s: &mut Stream<'a>) -> StreamResult<Token<'a>> {
+        let start = s.pos() - 9;
+
         let text = s.consume_bytes(|s, c| {
             !(c == b']' && s.starts_with(b"]]>"))
         });
 
         s.skip_string(b"]]>")?;
 
-        Ok(Token::Cdata(text))
+        let span = s.slice_back(start);
+
+        Ok(Token::Cdata { text, span })
     }
 
     fn parse_element_start(s: &mut Stream<'a>) -> Result<Token<'a>> {
@@ -872,8 +1027,11 @@ impl<'a> Tokenizer<'a> {
 
     // '<' Name (S Attribute)* S? '>'
     fn parse_element_start_impl(s: &mut Stream<'a>) -> StreamResult<Token<'a>> {
-        let (prefix, tag_name) = s.consume_qname()?;
-        Ok(Token::ElementStart(prefix, tag_name))
+        let start = s.pos() - 1;
+        let (prefix, local) = s.consume_qname()?;
+        let span = s.slice_back(start);
+
+        Ok(Token::ElementStart { prefix, local, span })
     }
 
     fn parse_close_element(s: &mut Stream<'a>) -> Result<Token<'a>> {
@@ -882,58 +1040,62 @@ impl<'a> Tokenizer<'a> {
 
     // '</' Name S? '>'
     fn parse_close_element_impl(s: &mut Stream<'a>) -> StreamResult<Token<'a>> {
+        let start = s.pos() - 2;
+
         let (prefix, tag_name) = s.consume_qname()?;
-        s.skip_ascii_spaces();
+        s.skip_spaces();
         s.consume_byte(b'>')?;
 
-        Ok(Token::ElementEnd(ElementEnd::Close(prefix, tag_name)))
+        let span = s.slice_back(start);
+
+        Ok(Token::ElementEnd { end: ElementEnd::Close(prefix, tag_name), span })
     }
 
     // Name Eq AttValue
     fn parse_attribute(s: &mut Stream<'a>) -> StreamResult<Token<'a>> {
-        s.skip_ascii_spaces();
+        s.skip_spaces();
+
+        if let Ok(c) = s.curr_byte() {
+            let start = s.pos();
 
-        if let Some(c) = s.get_curr_byte() {
             match c {
                 b'/' => {
                     s.advance(1);
                     s.consume_byte(b'>')?;
-                    return Ok(Token::ElementEnd(ElementEnd::Empty));
+                    let span = s.slice_back(start);
+                    return Ok(Token::ElementEnd { end: ElementEnd::Empty, span });
                 }
                 b'>' => {
                     s.advance(1);
-                    return Ok(Token::ElementEnd(ElementEnd::Open));
+                    let span = s.slice_back(start);
+                    return Ok(Token::ElementEnd { end: ElementEnd::Open, span });
                 }
                 _ => {}
             }
         }
 
-        let (prefix, name) = s.consume_qname()?;
+        let start = s.pos();
+
+        let (prefix, local) = s.consume_qname()?;
         s.consume_eq()?;
         let quote = s.consume_quote()?;
         let value = s.consume_bytes(|_, c| c != quote);
 
-        if value.to_str().contains('<') {
+        if value.as_str().contains('<') {
             return Err(StreamError::InvalidAttributeValue);
         }
 
         s.consume_byte(quote)?;
-        s.skip_ascii_spaces();
+        let span = s.slice_back(start);
 
-        Ok(Token::Attribute((prefix, name), value))
+        s.skip_spaces();
+
+        Ok(Token::Attribute { prefix, local, value, span })
     }
 
     fn parse_text(s: &mut Stream<'a>) -> Result<Token<'a>> {
         let text = s.consume_bytes(|_, c| c != b'<');
-
-        let mut ts = Stream::from(text);
-        // TODO: optimize
-        ts.skip_spaces();
-        if ts.at_end() {
-            Ok(Token::Whitespaces(text))
-        } else {
-            Ok(Token::Text(text))
-        }
+        Ok(Token::Text { text })
     }
 }
 
@@ -942,7 +1104,6 @@ impl<'a> Iterator for Tokenizer<'a> {
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.stream.at_end() || self.state == State::End {
-            self.state = State::End;
             return None;
         }
 
@@ -950,39 +1111,35 @@ impl<'a> Iterator for Tokenizer<'a> {
 
         if let Some(ref t) = t {
             match *t {
-                Ok(Token::ElementStart(..)) => {
-                    self.state = State::Attributes;
-                }
-                Ok(Token::ElementEnd(ref end)) => {
-                    match *end {
-                        ElementEnd::Open => {
-                            self.depth += 1;
-                        }
-                        ElementEnd::Close(..) => {
-                            if self.depth > 0 {
-                                self.depth -= 1;
-                            }
-                        }
-                        ElementEnd::Empty => {}
+                Ok(t) => match t {
+                    Token::ElementStart { .. } => {
+                        self.state = State::Attributes;
                     }
+                    Token::ElementEnd { ref end, .. } => {
+                        match *end {
+                            ElementEnd::Open => self.depth += 1,
+                            ElementEnd::Close(..) if self.depth > 0 => self.depth -= 1,
+                            _ => {}
+                        }
 
-                    if self.depth == 0 && !self.fragment_parsing {
-                        self.state = State::AfterElements;
-                    } else {
-                        self.state = State::Elements;
+                        if self.depth == 0 && !self.fragment_parsing {
+                            self.state = State::AfterElements;
+                        } else {
+                            self.state = State::Elements;
+                        }
                     }
-                }
-                Ok(Token::DtdStart(..)) => {
-                    self.state = State::Dtd;
-                }
-                Ok(Token::EmptyDtd(..)) | Ok(Token::DtdEnd) => {
-                    self.state = State::AfterDtd;
-                }
+                    Token::DtdStart { .. } => {
+                        self.state = State::Dtd;
+                    }
+                    Token::EmptyDtd { .. } | Token::DtdEnd { .. } => {
+                        self.state = State::AfterDtd;
+                    }
+                    _ => {}
+                },
                 Err(_) => {
                     self.stream.jump_to_end();
                     self.state = State::End;
                 }
-                _ => {}
             }
         }
 
